#!encoding=utf-8
import unittest
from ivystar.src.decorator import log
from ivystar.src.decorator import timer

class TestSetUp(unittest.TestCase):

    def setUp(self):
        print(">>> start test")


    def test_run(self):

        @log
        def fast(x, y):
            return x*y
        fast(3,5)

        @timer
        def fast(x, y):
            return x*y
        fast(3,5)

    def tearDown(self):
        print("<<< test end")

if __name__ == "__main__":
    unittest.main()
else:
    unittest.main()



#encoding=utf-8
# ================================================================
#   Copyright (C) 2020 ivystar Ltd. All rights reserved.
#   file: utils.py
#   mail: 2364939934@qq.com
#   date: 2020-07-02
#   describe: interface for mongodb
# ================================================================

from ivystar.src.dataio.base_helper import DataBaseHelper
import pymongo
from ivystar.src.conf import MONGO_IP
from ivystar.src.conf import MONGO_PORT

class MongoHelper(DataBaseHelper):
    """mongo client"""
    def __init__(self, ip, port):
        DataBaseHelper.__init__(self)
        myclient = pymongo.MongoClient("mongodb://%s:%s/"%(ip, port))
        for db in myclient.list_database_names():
            for coll in myclient[db].list_collection_names():
                print("%s.%s"%(db, coll))
        self.cli = myclient

if __name__ == "__main__":
    mh = MongoHelper(MONGO_IP, MONGO_PORT)



#!encoding=utf-8

'''
rsa加密解密模块,确保信息安全

rsa 加密原理,两个大质数,容易求积,却不容易拆开,用其中一个做私钥,另一个做公钥
公钥用来加密,私钥用来解密

作者: 秦海宁
时间: 2021-03-19
联系: 2364839934@qq.com

'''
import os
import rsa
from ivystar.src.conf import PROJECT_PATH # 项目路径
from ivystar.src.conf import SIGNATURE_FILE_PATH # rsa 签名文件路径

# 对api的签名机制进行验证：签名用私钥,验签用公钥
class RsaEncrypt:

  def __init__(self, sign_str):
    self.sign_str = sign_str

  def rsa_generate(self):
    """
    生成私钥和公钥并保存
    :return:
    """
    pubkey, privkey = rsa.newkeys(1024) # 生成公钥和私钥
    pub = pubkey.save_pkcs1() # 保存公钥
    with open(os.path.join(SIGNATURE_FILE_PATH, 'public.pem'), 'wb') as w_pub:
      w_pub.write(pub)
    pri = privkey.save_pkcs1() # 保存私钥
    with open(os.path.join(SIGNATURE_FILE_PATH, 'private.pem'), 'wb') as w_pub:
      w_pri.write(pri)
    return "保存成功"

  @classmethod #类方法
  def read_rsa(self):
    """
    读取公钥和私钥
    :return:
    """
    with open(os.path.join(SIGNATURE_FILE_PATH, 'public.pem'), 'rb') as publickfile:
      pub = publickfile.read()
      pubkey = rsa.PublicKey.load_pkcs1(pub)
    with open(os.path.join(SIGNATURE_FILE_PATH, 'private.pem'), 'rb') as privatefile:
      priv = privatefile.read()
      # print(pub)
      privkey = rsa.PrivateKey.load_pkcs1(priv)
    return pubkey, privkey

  def str_sign(self):
    privkey = self.read_rsa()[1] # 先将要加密的数据转成二进制
    str_encode = self.sign_str.encode() # 用私钥进行加密,并设置加密算法
    signature = rsa.sign(str_encode, privkey, 'SHA-256') # 签名加密算法可以更换比如：SHA-256
    # print(signature)
    return signature

  def sign_verify(self, signature):
    """
    验证签名是否正确,如果正确,则返回签名算法,否则返回验证失败
    :param signature:
    :return:
    """
    pubkey = self.read_rsa()[0]
    try:
      agl = rsa.verify(self.sign_str.encode(), signature, pubkey)
      # print(type(agl))
      print(agl) # 返回加密算法代表验签成功
      return True
    except rsa.VerificationError:
      print("验证失败")
      return False

# 对数据进行加密:加密用公钥,解密用私钥
class DataEncrypt:

  def __init__(self, data_str):
    self.data_str = data_str
    self.secret_key = RsaEncrypt.read_rsa()# 调用RsaEncrypt类的读取密钥对方法

  def data_encrypt(self):
    """
    用公钥对数据进行加密
    :return:
    """
    str_encrypt = rsa.encrypt(self.data_str.encode(), self.secret_key[0])
    print(str_encrypt) # 加密后看着像二进制,但有不太像,看不懂
    return str_encrypt

  def data_decrypt(self, encrypt):
      '''
      使用私钥对数据进行解密
      '''
      str = rsa.decrypt(encrypt, self.secret_key[1]).decode()
      print(str) # 返回加密前的数据
      return str

if __name__ == '__main__':
  # 验证签名机制
  input_txt = 'ivystar'
  sing_test = RsaEncrypt(input_txt)
  # sing_test.rsa_generate() # 生成密钥会覆盖当前密钥，请注意存档
  sing_test.sign_verify(sing_test.str_sign())

  # 验证加密解密机制
  data = DataEncrypt(input_txt)
  result = data.data_decrypt(data.data_encrypt())
  try:
      assert result == input_txt
      print("密码正确")
  except AssertException:
      print("密码错误")




#!encoding=utf-8

from bert4keras.models import build_transformer_model
from bert4keras.tokenizers import Tokenizer
import numpy as np
import sys
sys.path.append("/app/ivystar")
from ivystar.src.conf import BERT_MODEL_PATH
from ivystar.src.algorithm.predict.base_predict import BasePredictClass
from ivystar.src.algorithm.similar.calculate_similarity import pearson
print("BERT_MODEL_PATH", BERT_MODEL_PATH)
import os

#BERT_MODEL_PATH = "/home/qin/release/model/chinese_L-12_H-768_A-12"

'''
加载google预言模型，完成语言向量化
'''

class Word2VecPredict(BasePredictClass):
    def __init__(self, name):
        BasePredictClass.__init__(self, name)
        config_path = os.path.join(BERT_MODEL_PATH, "bert_config.json")
        checkpoint_path = os.path.join(BERT_MODEL_PATH, "bert_model.ckpt")
        dict_path = os.path.join(BERT_MODEL_PATH, "vocab.txt")

        self.kvs = {}
        self.kvs["config_path"] = config_path
        self.kvs["checkpoint_path"] = checkpoint_path
        self.kvs["dict_path"] = dict_path

    def load_model(self):
        tokenizer = Tokenizer(self.kvs["dict_path"], do_lower_case=True)  # 建立分词器
        model = build_transformer_model(self.kvs["config_path"], self.kvs["checkpoint_path"])  # 建立模型，加载权重
        self.model = model
        self.tokenizer = tokenizer

    def predict(self, texts, lenth=512, size=10):
        '''
        # 编码测试
        texts 等转换的文本
        lenth 每个句子保留多少个字符
        size 保留多少个句子
        '''
        texts = BasePredictClass.row_col_pendding(texts, lenth, size)
        token_ids_array, segment_ids_array = [], []
        for text in texts:
            token_ids, segment_ids = self.tokenizer.encode(text)
            token_ids_array.append(token_ids)
            segment_ids_array.append(segment_ids)
        res = self.model.predict([np.array(token_ids_array), np.array(segment_ids_array)])
        print(texts)
        print(res)
        return res

if __name__ == "__main__":
    mModel = Word2VecPredict("word2vec")
    mModel.load_model()
    res = mModel.predict(["中国" for i in range(1024)])




#!encoding=utf-8

class BasePredictClass(object):
    '''
    预测基类
    '''
    def __init__(self, name=""):
        self.name = name

    def load_model(self, **kws):
        pass

    def predict(self, texts, lenth, size):
        pass

    @staticmethod
    def row_col_pendding(texts, lenth, size):
        '''控制句子长度，句子数量'''
        return [text[:lenth] for text in texts][:size]





#!encoding=utf-8

from bert4keras.models import build_transformer_model
from bert4keras.tokenizers import Tokenizer
import numpy as np
from ivystar.src.conf import BERT_MODEL_PATH
import os

BERT_MODEL_PATH = "/home/qin/release/model/chinese_L-12_H-768_A-12"

config_path = os.path.join(BERT_MODEL_PATH, "bert_config.json")
checkpoint_path = os.path.join(BERT_MODEL_PATH, "bert_model.ckpt")
dict_path = os.path.join(BERT_MODEL_PATH, "vocab.txt")
print(dict_path)

tokenizer = Tokenizer(dict_path, do_lower_case=True)  # 建立分词器
model = build_transformer_model(config_path, checkpoint_path)  # 建立模型，加载权重

# 编码测试
token_ids, segment_ids = tokenizer.encode(u'语言模型')

print('\n ===== predicting =====\n')
print(model.predict([np.array([token_ids]), np.array([segment_ids])]))



import requests_html
!pip install requests_html
!pip install requests_html -i https://pypi.tuna.tsinghua.edu.cn/simple
!pip install requests_html -i https://pypi.tuna.tsinghua.edu.cn/simple
import requests_html
requests_html.HTML
requests_html.get("www.sohu.com")
from requests_html import HTMLSession
session = HTMLSession()
session.get("www.sohu.com")
session.get("http://www.sohu.com")
res = session.get("http://www.sohu.com")
res.html
res.html.html
res.html.html
type(res.html.html)
type(res.html.links)
res.html.links
res.html.absolute_link
res.html.absolute_links
res.html.links
res.html.absolute_links
res.html.html
re.findall("[\u4e00-\u9fa5]", res.html.html)
import re
re.findall("[\u4e00-\u9fa5]", res.html.html)
re.findall("[\u4e00-\u9fa5]*", res.html.html)
re.findall("[\u4e00-\u9fa5]+", res.html.html)
res.html.html
re.findall("[\u4e00-\u9fa5]+", res.html.html)
re.findall("[\u4e00-\u9fa5](?=(水|泥))+", res.html.html)
re.findall("[\u4e00-\u9fa5](?i<=(水|泥))+", res.html.html)
re.findall("[\u4e00-\u9fa5](?<=(水|泥))+", res.html.html)
re.findall("[\u4e00-\u9fa5])+", res.html.html)
re.findall("[\u4e00-\u9fa5]+)", res.html.html)
re.findall("[\u4e00-\u9fa5]+", res.html.html)
set(re.findall("[\u4e00-\u9fa5]+", res.html.html))
len(set(re.findall("[\u4e00-\u9fa5]+", res.html.html)))
len(re.findall("[\u4e00-\u9fa5]+", res.html.html))
len(re.findall("[\u4e00-\u9fa5a-zA-Z\d]+", res.html.html))
re.findall("[\u4e00-\u9fa5a-zA-Z\d]+", res.html.html)
re.findall("[\u4e00-\u9fa5a-zA-Z\d]+", res.html.html)
re.findall("[\u4e00-\u9fa5\d]+", res.html.html)
re.findall("(?=[\u4e00-\u9fa5\d])[^'])", res.html.html)
re.findall("(?=[\u4e00-\u9fa5\d])[^\'])", res.html.html)
re.findall("(?=([\u4e00-\u9fa5\d]))[^\'])", res.html.html)
re.findall("(?=([\u4e00-\u9fa5\d]))[^\'])", res.html.html)
re.findall("(?=([\u4e00-\u9fa5\d])).)", res.html.html)
re.findall("(?=([\u4e00-\u9fa5\d])).*)", res.html.html)
re.findall(".(?=([\u4e00-\u9fa5]))", res.html.html)
re.findall(".(?=([\u4e00-\u9fa5]))+", res.html.html)
re.findall(".+(?=([\u4e00-\u9fa5]))", res.html.html)
re.findall("(.(?=([\u4e00-\u9fa5])))+", res.html.html)
re.findall("(.(?=[\u4e00-\u9fa5]))+", res.html.html)
re.findall(".(?=[\u4e00-\u9fa5])+", res.html.html)
re.findall(".(?=[\u4e00-\u9fa5])*", res.html.html)
re.findall("[\u4e00-\u9fa5\da-zA-Z]+", res.html.html)
res.html.html
re.findall("[\u4e00-\u9fa5\da-zA-Z ]+", res.html.html)
re.findall("[\u4e00-\u9fa5\da-zA-Z ！，。、]+", res.html.html)
re.findall("[\u4e00-\u9fa5\da-zA-Z]+", res.html.html)
for i in re.findall("[\u4e00-\u9fa5\da-zA-Z]+", res.html.html):
    print(re.search("[\u4e00-\u9fa5\da-zA-Z]",i))
for i in re.findall("[\u4e00-\u9fa5\da-zA-Z]+", res.html.html):
    print(re.search("[\u4e00-\u9fa5\da-zA-Z]",i).index)
for i in re.findall("[\u4e00-\u9fa5\da-zA-Z]+", res.html.html):
    print(re.search("[\u4e00-\u9fa5\da-zA-Z]",i.index))
for i in re.findall("[\u4e00-\u9fa5\da-zA-Z]+", res.html.html):
    print(re.search("[\u4e00-\u9fa5\da-zA-Z]",i).index)
for i in re.findall("[\u4e00-\u9fa5\da-zA-Z]+", res.html.html):
    print(re.search("[\u4e00-\u9fa5\da-zA-Z]",i))
for i in re.findall("[\u4e00-\u9fa5\da-zA-Z]+", res.html.html):
    print(re.search("[\u4e00-\u9fa5]",i))
for i in re.findall("[\u4e00-\u9fa5\da-zA-Z]+", res.html.html):
    if (re.search("[\u4e00-\u9fa5]",i)):
        print(i)
for i in re.findall("[\u4e00-\u9fa5\da-zA-Z]+", res.html.html):
    if (re.search("[\u4e00-\u9fa5]",i)):
        print(i)
ii = []
for i in re.findall("[\u4e00-\u9fa5\da-zA-Z]+", res.html.html):
    if (re.search("[\u4e00-\u9fa5]",i)):
        print(i)
        ii.append(i)
import collections
collections.Counter(ii)
sorted(collections.Counter(ii))
sorted(collections.Counter(ii).items(), key=lambda x:x[1], reverse=True)
sorted(collections.Counter(ii).items(), key=lambda x:x[1], reverse=True)[:10]
sorted(collections.Counter(ii).items(), key=lambda x:x[1], reverse=True)[:20]
sorted(collections.Counter(ii).items(), key=lambda x:x[1], reverse=True)[:30]
sorted(collections.Counter(ii).items(), key=lambda x:x[1], reverse=True)[:100]
res.html.full_text
res.html.text
res.html.text[0]
res.html.text[1]
res.html.text
ii = []
for i in re.findall("[\u4e00-\u9fa5\da-zA-Z]+", res.html.text):
    if (re.search("[\u4e00-\u9fa5]",i)):
        print(i)
        ii.append(i)
ii
for _ in ii:
    if '睡觉' in _:
        print(_)
res.html.text
!grep -a -P '睡觉' res.html.text
cat res.html.text | grep -a -P '睡觉'
!cat res.html.text | grep -a -P '睡觉'
res.html.text | grep -a -P '睡觉'
ii = []
for i in re.findall("[\u4e00-\u9fa5\da-zA-Z]+", res.html.text):
    if '睡觉' in i:
        print(i)
    #if (re.search("[\u4e00-\u9fa5]",i)):
    #    print(i)
    #    ii.append(i)
res.html.full_text
res.html.full_text[0]
res.html.full_text
res.html.full_text.split("\n")
res.html.full_text.split("\\n")
res.html.full_text.split("\n")
res.html.full_text.split(u"\n")
[i.strip() for i in res.html.full_text.split(u"\n") if len(i.strip())>0]
[i.strip() for i in res.html.full_text.split(u"\n") if len(i.strip())>0]
[i.strip() for i in res.html.full_text.split(u"\n") if len(i.strip())>0]
[i.strip() for i in res.html.full_text.split(u"\n") if len(i.strip())>0]
[i.strip() for i in res.html.full_text.split(u"\n") if len(i.strip())>0]
[i.strip() for i in res.html.full_text.split(u"\n") if len(i.strip())>3]
[i.strip() for i in res.html.full_text.split(u"\n") if len(i.strip())>3]
res.html.url
res.html.xpath()
res.html.encoding
res.html.time
res.html.page
res.html.page()
res.html.raw_html
[i.strip() for i in res.html.text.split(u"\n") if len(i.strip())>3]
[i.strip() for i in res.html.full_text.split(u"\n") if len(i.strip())>3]
len([i.strip() for i in res.html.full_text.split(u"\n") if len(i.strip())>3])
len([i.strip() for i in res.html.text.split(u"\n") if len(i.strip())>3])
b = [i.strip() for i in res.html.text.split(u"\n") if len(i.strip())>3]
a = [i.strip() for i in res.html.full_text.split(u"\n") if len(i.strip())>3]
b in a
for i in b:
    if i in a:
        pass
    print(i)
ps -af
ls -rlth
ps -af
%hist -f tmp.py



#!encoding=utf-8

from ivystar.src.web.http_fetch import HttpRequest
from ivystar.src.web.http_analysis import HttpAnalyze
from ivystar.src.dataio.mongo_helper import MongoHelper
from ivystar.src.conf import MONGO_IP
from ivystar.src.conf import MONGO_PORT
from ivystar.src.conf import SPIDER_LINK
from ivystar.src.conf import SPIDER_SUCC
from ivystar.src.conf import SPIDER_RESULT
from ivystar.src.decorator import prilog
import multiprocessing
from multiprocessing import Pool

global p
global l
global mongo_cli
global ha
ha = HttpAnalyze()
mongo_cli = MongoHelper(MONGO_IP, MONGO_PORT) 
l = multiprocessing.Lock()

class UrlSpider(HttpRequest):

    def __init__(self):
        HttpRequest.__init__(self)

    def run_task(self, urls, num):
        '''
        单机运行多线程抓取任务
        '''
        global p #p不能作为实例变量在进程间传递和序列化。
        p = Pool(num)
        tmp_urls = []
        for url in urls:
            tmp_urls.append(url)
            if len(tmp_urls) == num:
                p.map_async(self.task, tuple(tmp_urls), callback=self.back_func, error_callback=self.back_func_err)
                tmp_urls = []
        p.close()
        p.join()

    def task(self, url):
        global ha
        res = ha(url)
        return res

    def back_func(self, values):
        global l 
        global mongo_cli
        l.acquire()
        for value in values:
            for url in value["links"]:
                mongo_cli.cli["db"][SPIDER_LINK].insert({"url":url})
            mongo_cli.cli["db"][SPIDER_SUCC].insert({"url":value["url"]})
            mongo_cli.cli["db"][SPIDER_RESULT].insert({"url":value["url"], "texts":value["texts"]})
        l.release()
        print("back_func:", values)

    def back_func_err(self, value):
        print("back_func_err:", value)

if __name__ == "__main__":
    mus = UrlSpider()
    NUM = 3
    EPOCH = 10 # 扫描轮数

    ''' 加入原始网站 '''
    for source_url in ["http://www.chinaai.com", "http://www.iteye.com", "https://www.new.qq.com"]:
        mongo_cli.cli["db"][SPIDER_LINK].update_one({"url":source_url}, {"$set":{"url":source_url}}, True)

    ''' 开始扫描 '''
    print([i for i in mongo_cli.cli["db"][SPIDER_LINK].find()])
    for i in range(10):
      urls = []
      for url in mongo_cli.cli["db"][SPIDER_LINK].find({},{"_id":0}):
        if 0 == mongo_cli.cli["db"][SPIDER_SUCC].find({"url":url["url"]},{"_id":0}).count():
          if 'chinaai' in url['url'] or 'iteye' in url["url"] or 'new.qq' in url["url"]:
            urls.append(url["url"])
            print(urls)
            if len(urls) == NUM:
              print(urls)
              mus.run_task(urls, NUM)
              urls = []




#!encoding=utf-8

import numpy as np
from sklearn.metrics import pairwise_distances
from sklearn.metrics.pairwise import cosine_similarity
from scipy.spatial.distance import cosine

'''
# 计算欧几里德距离：
'''
def euclidean(p, q):
    '''
    计算向量间欧几里得距离，并归一化
    向量不等长，则按照短的那个尺寸计算
    '''
    short_lenth = min(len(p), len(q))
    e = sum([(p[i] - q[i]) ** 2 for i in range(short_lenth)])
    return 1 / (1 + e ** .5)

def pearson(*args):
    A = np.array(args)
    dist_out = 1-pairwise_distances(A, metric="cosine")
    return dist_out

def manhattan(p, q):
    '''
    #计算曼哈顿距离：
    '''
    short_lenth = min(len(p), len(q))
    n = short_lenth
    vals = range(n)
    distance = sum(abs(p[i] - q[i]) for i in vals)
    return distance

def jaccrad(source, target):
    '''
    jaccrad系数， 参数为0，1二元值时的相似性计算
    '''
    up = set(source) & set(target)
    down = (set(source) | set(target)) - up
    return float(len(up) / len(down))

if __name__ == "__main__":
    a,b,c = [1,2,3,4,5,6], [1,2,3,4,5,7], [4,5,6,7,8,9]
    print(jaccrad(a,b))
    print(pearson(a,b,c))
    print(cosine_similarity([a],[b]))
    print(euclidean(a,b))
    print(manhattan(a,b))





#!encoding=utf-8

class DataBaseHelper(object):
    '''
    数据库接口基类
    '''
    def __init__(self):
        pass

    def connect(self):
        '''
        建立连接数据库
        '''
        pass

    def exec(self):
        '''
        运行指令
        '''
        pass



#!encoding=utf-8
"""
多线程 request http
"""

import multiprocessing
from multiprocessing import Pool
import sys

# 任务池Pool
global p
global l # nosql没有事务，使用线程锁保证多线程安全
l = multiprocessing.Lock()

class HttpRequest(object):
    '''
    多线程访问http类
    '''
    def __init__(self):
        print('init HttpRequest')

    def run_task(self, urls, num):
        '''
        单机运行多线程抓取任务
        '''
        global p #p不能作为实例变量在进程间传递和序列化。
        p = Pool(num)
        tmp_urls = []
        for url in urls:
            tmp_urls.append(url)
            if len(tmp_urls) == num:
                p.map_async(self.task, tuple(tmp_urls), callback=self.back_func, error_callback=self.back_func_err)
                tmp_urls = []
        p.close()
        p.join()

    def task(self, tmp_urls):
        '''
        处理单条url
        '''
        print('run task')
        return [ValueError,'a']
    
    def back_func(self, values):
        global l
        l.acquire()
        '''
        处理返回值
        '''
        for value in values:
            print(value)
        l.release()
    
    def back_func_err(self, values):
        '''
        处理返回异常
        '''
        print('run back_func_err:', type(values), values)

if __name__ == "__main__":
    mHttpHandler = HttpRequest()
    mHttpHandler.run_task(['www.baidu.com','www.sohu.com','www.sina.com'], 3)





#!encoding=utf-8

# 将路径配置入环境变量

import os

PROJECT_PATH = os.getenv("PROJECT_PATH")
SIGNATURE_FILE_PATH = os.getenv("SIGNATURE_FILE_PATH")
BERT_MODEL_PATH = os.getenv("BERT_MODEL_PATH")
MONGO_IP= os.getenv("MONGO_IP")
MONGO_PORT= os.getenv("MONGO_PORT")
SPIDER_LINK = os.getenv("SPIDER_LINK")
SPIDER_SUCC = os.getenv("SPIDER_SUCC")
SPIDER_RESULT = os.getenv("SPIDER_RESULT")

#print(PROJECT_PATH, SIGNATURE_FILE_PATH, BERT_MODEL_PATH, MONGO_IP, MONGO_PORT)
# 'bert/chinese_L-12_H-768_A-12/bert_config.json'



#!encoding=utf-8
"""
多线程 request http
"""

import sys
import re
import json
import src
from src.regular_utils import AddressPreHandlerUtil
import multiprocessing
from multiprocessing import Pool
import os
import traceback
import time
import pandas as pd
import pdb
import pymongo
import redis
import argparse
from src.decorator.time_decorator import time_out
from src.decorator.time_decorator import timeout_callback


class HttpRequest(object):
    '''
    多线程访问http类
    '''
    def __init__(self):
        pass

    def run_task(self, task, urls, back_func, back_func_err, num):
        '''
        单机运行多线程抓取任务
        '''
        self.p = Pool(num)
        self.p.map_async(self.task, tuple(self.urls), callback=self.back_func, error_callback=self.back_func_err)
        self.p.close()
        self.p.join()

    def task():
        '''
        处理单条url
        '''
        pass
    
    def back_func():
        '''
        处理返回值
        '''
        pass
    
    def back_func_err():
        '''
        处理返回异常
        '''
        pass

if __name__ == "__main__":
    try:
        mHttpHandler = HttpRequest()
        mHttpHandler.run_task(task, urls, back_func, back_func_err, num)
    except:
        traceback.print_exc()
    finally:
        sys.exit(0)

def handler_line(line, outf=None, level="PICK"):
    """
    :function :处理单行
    :parameter outf: 输出mongo
    :parameter line: 输出输入文本
    """
    try:
        global lock
        res = src.baidu_helper.fetch_one(line)
        res_ = json.loads(res)
        res_['source'] = line
        return res_
    except ValueError:
        print("what's up?")
        traceback.print_exc()
        return {"status": "handler_line: Value Error"}
    except ValueError:
        print("what's up?")
        traceback.print_exc()
        return {"status": "handler_line: any other Error"}

'''
返回处理参数
'''
def back_func(values): # 多进程执行完毕会返回所有的结果的列表
    __ = []
    for _ in values:
        try:
            if _ == None:
                continue
            _['action'] = 'suggestion'
            #rediscli.set(_["source"], 1)
            #print(_)
            __.append(_)
        except:
            print("__err__>> ", values)
            traceback.print_exc()
        finally:
            print("__final__", _["source"])
    global lock
    lock.acquire()
    mycoll_out.insert_many(__)
    lock.release()

'''
错误处理回调函数
'''
def back_func_err(values): # 多进程执行完毕会返回所有错误的列表
    print('error', values)

def handler():
    """ 读入所有的地址 """
    try:
        address_words_set = set()
        address_words = []
        NUM = 17
        p = Pool(NUM)
        #mongocli = pymongo.MongoClient("172.20.0.10", 27017)
        mongocli = myclient
        for item in mongocli["db"][coll_in].aggregate([{"$project":{field_name:1,"_id":0}},{"$limit":290000}]):
            if not field_name in item:
                continue
            #_ = item.get("address", "")
            #_ = item["field3"]
            if type(item[field_name]) == str:
                if len(item[field_name]) > 0:
                    address_words_set.add(aphu.sub(item[field_name]))
                    #print(item)
        for _ in list(address_words_set):
            try:
                address_words.append(src.baidu_helper.build_place_v2_suggestion(_, city_name))
            except AttributeError:
                traceback.print_exc()
            except:
                traceback.print_exc()
        matrix_fetch = []
        urls = []
        for index, address_word in enumerate(address_words):
            urls.append(address_word)
            if len(urls) == NUM:
                p.map_async(handler_line, tuple(urls), callback=back_func, error_callback=back_func_err)
                urls = []
        if len(urls) > 0:
            p.map_async(handler_line, tuple(urls), callback=back_func, error_callback=back_func_err)
            matrix_fetch = []
        p.close()
        p.join()
        print("文件处理完毕")
    except FileNotFoundError:
        print("FileNotFoundError")


"""
运行爬取
python3 example_baidu_suggestion.py --input_coll sd_jn --city_name 济南市 --output_coll sd_jn_search_0227 --field address
#python3 example_baidu_suggestion.py --input_coll wj_jq_data_1126 --city_name 吴江区 --output_coll wj_jq_data_1126_suggestion --filed field3
"""




#!encoding=utf-8
"""
html内容解析
"""
import sys
import requests_html
from requests_html import HTMLSession
import re
import pdb

class HttpAnalyze(object):
    '''
    http 访问结果解析
    '''
    def __init__(self):
        print('init HttpAnalyze')
        session = HTMLSession()
        self.session = session

    def get_url(self, url):
        '''
        读取url内容
        '''
        res = self.session.get(url)
        return res

    def text_filter(self, res):
        '''
        过滤出带有汉字的句子
        '''
        texts_a = [i.strip() for i in res.html.full_text.split(u"\n") if len(i.strip())>0 and re.search("^[\u4e00-\u9fa5]", i.strip())]
        texts_b = [i.strip() for i in res.html.text.split(u"\n") if len(i.strip())>0 and re.search("^[\u4e00-\u9fa5]", i.strip())]
        texts_c = list(set(texts_a + texts_b))
        return texts_c

    def links_filter(self, res):
        '''
        返回所有的链接
        '''
        target_links = []
        links = list(res.html.links) + list(res.html.absolute_links)
        for _ in links:
            if _[:4] == "http":
                target_links.append(_)
            else:
                target_links.append(res.html.url[:-1]+_)
        return list(set(target_links))

    def __call__(self, url):
        content = self.get_url(url)
        texts = self.text_filter(content)
        links = self.links_filter(content)
        page_info = {}
        page_info["texts"] = texts
        page_info["links"] = links
        page_info["url"] = url
        return page_info

if __name__ == "__main__":
    ha = HttpAnalyze()
    response = ha("http://www.triplet.com.cn")
    print(response)
